#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
#endregion

namespace SlidingBlockPuzzle {

	public class BlocksControl {
		readonly Blocks blocks = new Blocks();
		readonly Randomizer randomizer;
		readonly Slider slider;

		public event Action<int> CurrentIndexChanged;

		Point current;
		int currentIndex;

		public BlocksControl() {
			randomizer = new Randomizer(blocks);
			slider = new Slider(blocks);
			slider.Moved += TuneCursorPosition;
		}

		void TuneCursorPosition(int index, int x, int y) {
			if (blocks[index] == Blocks.EmptyBlock) {
				switch (Math.Sign(x)) {
				case +1:
					Left();
					break;
				case -1:
					Right();
					break;
				}
				switch (Math.Sign(y)) {
				case +1:
					Up();
					break;
				case -1:
					Down();
					break;
				}
			}
		}
		public Blocks Blocks { get { return blocks; } }
		public Slider Slider { get { return slider; } }
		public void Clear() {
			current.X = 0;
			current.Y = 0;
			Change();
		}
		public void Left() {
			do {
				if (--current.X < 0)
					current.X += Blocks.Width;
				ComputeCurrentIndex();
			} while (blocks.EmptyIndex == CurrentIndex);
			Change();
		}
		public void Right() {
			do {
				if (++current.X >= Blocks.Width)
					current.X -= Blocks.Width;
				ComputeCurrentIndex();
			} while (blocks.EmptyIndex == CurrentIndex);
			Change();
		}
		public void Up() {
			do {
				if (--current.Y < 0)
					current.Y += Blocks.Height;
				ComputeCurrentIndex();
			} while (blocks.EmptyIndex == CurrentIndex);
			Change();
		}
		public void Down() {
			do {
				if (++current.Y >= Blocks.Height)
					current.Y -= Blocks.Height;
				ComputeCurrentIndex();
			} while (blocks.EmptyIndex == CurrentIndex);
			Change();
		}
		public void Do() {
			if (slider.Slide(CurrentIndex)) {
			} else {
			}
		}
		public void Randomize() {
			randomizer.Randomize(100);
		}
		void ComputeCurrentIndex() {
			currentIndex = current.X + current.Y * Blocks.Width;
		}
		void Change() {
			if (CurrentIndexChanged != null)
				CurrentIndexChanged(currentIndex);
		}
		public int CurrentIndex { get { return currentIndex; } }
	}

}
