#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace SlidingBlockPuzzle {

	public class BlockVisual {
		Vector2 position;
		Vector2 goal;
		Rectangle sourceRectangle;
		Color color = Color.White;

		public Vector2 GoalPosition {
			get { return goal; }
			set { goal = value; }
		}
		public Vector2 Position {
			get { return position; }
			set { position = value; }
		}
		public Rectangle SourceRectangle {
			get { return sourceRectangle; }
			set { sourceRectangle = value; }
		}
		public Color Color {
			get { return color; }
			set { color = value; }
		}
		public void Update() {
			Vector2.Lerp(ref position, ref goal, 0.1f, out position);
		}
	}

	public class Visual : DrawableGameComponent {
		SpriteBatch sprites;
		Texture2D texture;
		readonly ContentManager content;
		readonly BlocksControl control;

		readonly BlockVisual[] blocks = new BlockVisual[Blocks.NumberBlocks];

		const int Scale = 128;
		const int BlockPixels = 128;

		Vector2 position = new Vector2(
			800 * 0.5f - BlockPixels * Blocks.Width * 0.5f,
			600 * 0.5f - BlockPixels * Blocks.Height * 0.5f);

		public Visual(Game game, ContentManager content, BlocksControl control)
			: base(game) {
			this.content = content;
			this.control = control;
			for (int i = 0; i < blocks.Length; i++) {
				blocks[i] = new BlockVisual();
			}

			control.CurrentIndexChanged += UpdateTarget;
			control.Slider.Moved += Slided;
			control.Blocks.Completed += Completed;

			Clear();
		}

		void Completed(object sender, EventArgs e) {
			for (int i = 0; i < blocks.Length; i++) {
				blocks[i].Color = Color.White;
			}
		}

		void Slided(int index, int x, int y) {
#if DEBUG
			Console.WriteLine("index={0} x={1} y={2} {3}", index, x, y, control.Blocks[index]);
#endif
			blocks[control.Blocks[index]].GoalPosition += new Vector2(x, y) * Scale;
		}

		void UpdateTarget(int index) {
			for (int i = 1; i < blocks.Length; i++) {
				blocks[i].Color = Color.White;
			}
			blocks[control.Blocks[index]].Color = Color.Red;
		}
		protected override void LoadGraphicsContent(bool loadAllContent) {
			if (loadAllContent) {
				texture = content.Load<Texture2D>("content/texture/SlidingBlockPuzzle");
				sprites = new SpriteBatch(GraphicsDevice);
			}
			base.LoadGraphicsContent(loadAllContent);
		}

		public void Clear() {
			int i = 0;
			for (int y = 0; y < Blocks.Height; y++) {
				for (int x = 0; x < Blocks.Width; x++) {
					int index = control.Blocks[i++];
					BlockVisual visual = blocks[index];
					visual.GoalPosition = visual.Position
						= position + new Vector2(x * Scale, y * Scale);
					index += Blocks.NumberBlocks - 1;
					index %= Blocks.NumberBlocks;
					visual.SourceRectangle = new Rectangle(
						(index % Blocks.Width) * BlockPixels,
						(index / Blocks.Width) * BlockPixels,
						BlockPixels - 1,
						BlockPixels - 1);
				}
			}
#if DEBUG
			blocks[0].Color = new Color(new Vector4(1, 1, 1, 0.25f));
#else
			blocks[0].Color = new Color(new Vector4(1, 1, 1, 0));
#endif
			UpdateTarget(0);
			control.Blocks.Evaluate();
		}
		public override void Update(GameTime gameTime) {
			for (int i = 0; i < Blocks.NumberBlocks; i++) {
				blocks[i].Update();
			}
			base.Update(gameTime);
		}
		public override void Draw(GameTime gameTime) {
			sprites.Begin();
			for (int i = 0; i < Blocks.NumberBlocks; i++) {
				sprites.Draw(texture, blocks[i].Position, blocks[i].SourceRectangle, blocks[i].Color);
			}
			sprites.End();
			base.Draw(gameTime);
		}
	}

}
